;-------------------------------------------------------------------
; DKRZ NCL Example:  read_data_and_grid_corner_write_data_and_grid_corners.ncl
;
; Description:		Write grid corners and data to new netCDF file
;
; Files:			SAFRAN_PR_19890101net3.nc		- data file PRCP, time, lat, lon
;					grid_corners_SAFRAN.nc			- grid corners lat, lon, 4
;					SAFRAN_PR_19890101net3_added_grid_corners.nc - new output file
;
; 27.06.17  meier-fleischer(at)dkrz.de
;-------------------------------------------------------------------
begin
;-- file names
  corners         = "grid_corners_SAFRAN.nc"
  data            = "SAFRAN_PR_19890101net3.nc"
  outf            = "SAFRAN_PR_19890101net3_added_grid_corners.nc"

;-- open files
  fc              =  addfile(corners,"r")
  fd              =  addfile(data,"r")
        
;-- read variables        
  lat             =  fd->lat                        ;-- latitude
  lat@bounds      = "grid_corner_lat"                  ;-- add bounds attribute to lat
  lon             =  fd->lon                        ;-- longitude
  lon@bounds      = "grid_corner_lon"                  ;-- add bounds attribute to lon
  time            =  fd->time                       ;-- time
  alti            =  fd->Altitude                   ;-- variable Altitude
  prcp            =  fd->PRCP                       ;-- variable PRCP
  altiname        = "Altitude"        
  prcpname        = "PRCP"        

;-- read grid corners = vertices       
  grid_corner_lat =  fc->grid_corner_lat            ;-- grid_corner_lat(y, x, grid_corners)
  grid_corner_lon =  fc->grid_corner_lon            ;-- grid_corner_lon(y, x, grid_corners)
  grid_corners    =  dimsizes(grid_corner_lat(0,0,:)) ;-- vertices (grid_corners) = 4

;-- get the dimension names and sizes
  dimNames         =  getvardims(prcp)              ;-- retrieve the dimension names from variable PRCP
  ndims            =  dimsizes(getvardims(prcp))
  dnames           =  new(ndims+1,string)           ;-- we want to add dimension vertices
  dnames(:ndims-1) =  dimNames                      ;-- what we already got
  dnames(ndims)    = "grid_corners"                 ;-- add vertices 
  
  x_ind            =  ind(dimNames .eq. "x")        ;-- which index of x in dimNames
  y_ind            =  ind(dimNames .eq. "y")        ;-- which index of y in dimNames
  vardims          =  dimsizes(prcp)                ;-- retrieve dimension numbers of variable PRCP 
  x                =  vardims(x_ind)                ;-- number of x-values
  y                =  vardims(y_ind)                ;-- number of y-values

;-- create new netCDF file        
  system("rm -rf "+outf)                            ;-- delete file if already exists
  fout = addfile(outf,"c")                          ;-- open new file
  setfileoption(fout,"DefineMode",True)             ;-- explicitly declare file definition mode
  
;-- create global attributes of the file        
  fAtt               =  True                        ;-- assign file attributes
  fAtt@title         = "NCL add grid corners to file"
  fAtt@source_file   =  data+" and "+corners
  fAtt@Conventions   = "CF 1.5"   
  fAtt@creation_date =  systemfunc ("date")        
  fAtt@history       = "NCL script: read_data_and_grid_corner_write_data_and_grid_corners.ncl" 
  fAtt@comment       = "Combine two netCDF files: data file and grid corners file"       
  fileattdef(fout,fAtt)                             ;-- write file attributes to new file

;-- predefine the coordinate variables and their dimensionality
  dimSizes = (/ -1   , y,   x , grid_corners/)          ;-- dimension sizes
  dimUnlim = (/ True , False, False, False/)        ;-- dimension types
  filedimdef(fout,dnames,dimSizes,dimUnlim)         ;-- write dimensions to new file

;-- predefine the the dimensionality of the variables to be written out
  varNames1D   = "time"                             ;-- 1D variable names
  varTypes1D   =  typeof(time)                      ;-- 1D variable dimension types 
  varDims1D    =  getvardims(time)                  ;-- 1D retrieve variable dimensions
  
  varNames2D   = (/"lat","lon",altiname/)           ;-- 2D variable names
  varTypes2D   = (/typeof(lat),typeof(lon),typeof(alti)/);-- 2D variable dimension types 
  varDims2D    = getvardims(lat)
  
  varNames3D   =  prcpname                          ;-- 3D variable name
  varTypes3D   =  typeof(prcp)                      ;-- 3D variable dimension types
  varDims3D    =  getvardims(prcp)                  ;-- 3D retrieve variable dimensions

  varNamesVert = (/"grid_corner_lat", "grid_corner_lon"/) ;-- 3D variable name
  varTypesVert = (/typeof(grid_corner_lat), typeof(grid_corner_lon)/);-- 3D variable dimension types
  varDimsVert  =  getvardims(grid_corner_lat)

  filevardef(fout, varNames1D  , varTypes1D,   varDims1D)                          
  filevardef(fout, varNames2D  , varTypes2D,   varDims2D)
  filevardef(fout, varNames3D  , varTypes3D,   varDims3D)
  filevardef(fout, varNamesVert, varTypesVert, varDimsVert)
 
;-- copy attributes associated with each variable to the file
  filevarattdef(fout,"time",         time)          ;-- copy time attributes
  filevarattdef(fout,"lat",          lat)           ;-- copy lat attributes
  filevarattdef(fout,"grid_corner_lat", grid_corner_lat)  ;-- copy grid_corner_lat attributes
  filevarattdef(fout,"lon",          lon)           ;-- copy lon attributes
  filevarattdef(fout,"grid_corner_lon", grid_corner_lon)  ;-- copy grid_corner_lon attributes
  filevarattdef(fout, altiname,      alti)          ;-- copy variable attributes
  filevarattdef(fout, prcpname,      prcp)          ;-- copy variable attributes

  setfileoption(fout,"DefineMode",False)            ;-- explicitly exit file definition mode

;-- "(/", "/)" syntax strips off all attributes which we have already defined for the file variables
  fout->time            =  (/time/)                 ;-- write time to new netCDF file
  fout->lat             =  (/lat/)                  ;-- write lat to new netCDF file
  fout->grid_corner_lat =  (/grid_corner_lat/)      ;-- write lev to new netCDF file
  fout->lon             =  (/lon/)                  ;-- write lon to new netCDF file
  fout->grid_corner_lon =  (/grid_corner_lon/)      ;-- write lev to new netCDF file
  fout->$altiname$      =  (/alti/)                 ;-- write variable to new netCDF file
  fout->$prcpname$      =  (/prcp/)                 ;-- write variable to new netCDF file
  
end
